#!/usr/bin/env python3
#
#  artix.buildtree.py - Get Artix PKGBUILD tree
#

import argparse
import os
import shutil
import subprocess
from pathlib import Path
from pygit2 import clone_repository

debug = ""


def setup_options():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--tree-dir",
        default="/media/Repo/Trees",
        help="dir where the trees are created"
    )
    opts = parser.parse_args()
    return opts


def git_pull_in_dir(path_to_dir):
    original_dir = os.getcwd()
    os.chdir(path_to_dir)
    print(f"Updating {pkg}")
    exit_code = os.system('git pull 2> /dev/null')
    if exit_code:
        print(f"git update on {pkg} failed")
    else:
        processed.append(pkg)
    os.chdir(original_dir)
    return exit_code


def get_base_pkg():
    try:
        output = subprocess.check_output(["expac", "-S1", "%e", orig_pkg], universal_newlines=True)
    except:
        return False
    if output:
        pkgbase = output.strip()
        print(f"Split package, pkgbase is: {pkgbase}")
    else:
        pkgbase = False
    return pkgbase


def update_or_get_artix_pkg(this_pkg):
    if dir_path.exists() and dir_path.is_dir():
        no_git = git_pull_in_dir(repo_dir + "/" + this_pkg)
        if not no_git:
            return True
        else:
            print(f"Removing non git dir for {this_pkg}")
            shutil.rmtree(dir_path)
            try:
                clone_repository(gitea_link, repo_dir + "/" + this_pkg)
                print(f"Cloned {this_pkg}")
                processed.append(pkg)
                return True
            except:
                return False
    else:
        try:
            clone_repository(gitea_link, repo_dir + "/" + this_pkg)
            print(f"Cloned {this_pkg}")
            processed.append(pkg)
            return True
        except:
            return False


def check_oddities(this_pkg):
    match this_pkg:
        case "dvd+rw-tools":
            return "dvd-rw-tools"
        case "slim-runit":
            return False
        case "gnome-bluetooth":
            return False
        case "lib32-libmm-glib":
            return False
    if "+" in this_pkg:
        return this_pkg.replace("+", "plus")
    return this_pkg


def check_dir_oddites(this_pkg):
    main_pkg_chk = len(this_pkg.split("-"))
    if main_pkg_chk > 1:
        prefix = this_pkg.split("-")[0]
        match prefix:
            case "perl":
                return "perl"
        match prefix:
            case "python":
                return "python"
        match prefix:
            case "ruby":
                return "ruby"
    return this_pkg[:1]


options = setup_options()

dist = "artix"
repos = ["system", "world", "galaxy", "lib32"]
t_dir = options.tree_dir

if not os.path.exists(t_dir):
    os.makedirs(t_dir)

d_dir = os.path.join(Path(t_dir, dist))
if not os.path.exists(d_dir):
    os.makedirs(d_dir)

for repo in repos:
    repo_dir = os.path.join(Path(d_dir, repo))
    if not os.path.exists(os.path.join(Path(t_dir, dist, repo))):
        os.makedirs(os.path.join(Path(t_dir, dist, repo)))


fail = []
for repo in repos:
    repo_dir = os.path.join(os.path.join(Path(t_dir), dist, repo))
    pkg_list = os.popen("pacman -Sl " + repo + "| awk '{print $2}'").read().splitlines()
    processed = []
    for pkg in pkg_list:
        if pkg != debug and debug != "":
            continue
        else:
            debug = ""
        print(f"\nProcessing {pkg} from {repo}")
        orig_pkg = pkg
        pkg = check_oddities(pkg)
        if not pkg:
            continue
        #git_dir = check_dir_oddites(pkg)
        gitea_link = "https://gitea.artixlinux.org/packages" + "/" + pkg
        dir_path = Path(repo_dir + "/" + pkg)
        if update_or_get_artix_pkg(pkg):
            continue
        else:
            is_split_pkg = get_base_pkg()
            if is_split_pkg in processed:
                print(f"pkgbase {is_split_pkg} already processed")
                continue
            if is_split_pkg:
                dir_path = Path(repo_dir + "/" + is_split_pkg)
                # git_dir = check_dir_oddites(is_split_pkg)
                # gitea_link = "https://gitea.artixlinux.org/packages" + git_dir + "/" + is_split_pkg
                gitea_link = "https://gitea.artixlinux.org/packages" + "/" + is_split_pkg
                update_or_get_artix_pkg(is_split_pkg)
            else:
                fail.append(pkg)
    if fail:
        print(f"Failed to retrive ", end="")
        print("\n".join(str(x) for x in fail))
